from __future__ import absolute_import

__all__ = ['BaseLoader', 'SafeLoader', 'Loader', 'RoundTripLoader']

try:
    from .reader import *
    from .scanner import *
    from .parser_ import *
    from .composer import *
    from .constructor import *
    from .resolver import *
except (ImportError, ValueError):  # for Jython
    from ruamel.yaml.reader import *
    from ruamel.yaml.scanner import *
    from ruamel.yaml.parser_ import *
    from ruamel.yaml.composer import *
    from ruamel.yaml.constructor import *
    from ruamel.yaml.resolver import *



class BaseLoader(Reader, Scanner, Parser, Composer, BaseConstructor,
                 BaseResolver):
    def __init__(self, stream):
        Reader.__init__(self, stream)
        Scanner.__init__(self)
        Parser.__init__(self)
        Composer.__init__(self)
        BaseConstructor.__init__(self)
        BaseResolver.__init__(self)


class SafeLoader(Reader, Scanner, Parser, Composer, SafeConstructor, Resolver):
    def __init__(self, stream):
        Reader.__init__(self, stream)
        Scanner.__init__(self)
        Parser.__init__(self)
        Composer.__init__(self)
        SafeConstructor.__init__(self)
        Resolver.__init__(self)


class Loader(Reader, Scanner, Parser, Composer, Constructor, Resolver):

    def __init__(self, stream):
        Reader.__init__(self, stream)
        Scanner.__init__(self)
        Parser.__init__(self)
        Composer.__init__(self)
        Constructor.__init__(self)
        Resolver.__init__(self)


class RoundTripLoader(Reader, RoundTripScanner, Parser,
                      Composer, RoundTripConstructor, Resolver):
    def __init__(self, stream):
        Reader.__init__(self, stream)
        RoundTripScanner.__init__(self)
        Parser.__init__(self)
        Composer.__init__(self)
        RoundTripConstructor.__init__(self)
        Resolver.__init__(self)



# coding: utf-8

"""
comment testing is all about roundtrips
these can be done in the "old" way by creating a file.data and file.roundtrip
but there is little flexibility in doing that

but some things are not easily tested, eog. how a
roundtrip changes

"""

import pytest

import ruamel.yaml
from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump


class TestComments:
    def test_no_end_of_file_eol(self):
        """not excluding comments caused some problems if at the end of
        the file without a newline. First error, then included \0 """
        x = """\
        - europe: 10 # abc"""
        round_trip(x, extra='\n')
        with pytest.raises(AssertionError):
            round_trip(x, extra='a\n')

    def test_no_comments(self):
        round_trip("""
        - europe: 10
        - usa:
          - ohio: 2
          - california: 9
        """)

    def test_round_trip_ordering(self):
        round_trip("""
        a: 1
        b: 2
        c: 3
        b1: 2
        b2: 2
        d: 4
        e: 5
        f: 6
        """)

    def test_complex(self):
        round_trip("""
        - europe: 10 # top
        - usa:
          - ohio: 2
          - california: 9 # o
        """)

    def test_dropped(self):
        round_trip("""
        # comment
        scalar
        ...
        """, "scalar\n...\n")

    def test_main_mapping_begin_end(self):
        round_trip("""
        # C start a
        # C start b
        abc: 1
        ghi: 2
        klm: 3
        # C end a
        # C end b
        """)

    def test_main_mapping_begin_end_items_post(self):
        round_trip("""
        # C start a
        # C start b
        abc: 1      # abc comment
        ghi: 2
        klm: 3      # klm comment
        # C end a
        # C end b
        """)

    def test_main_sequence_begin_end(self):
        round_trip("""
        # C start a
        # C start b
        - abc
        - ghi
        - klm
        # C end a
        # C end b
        """)

    def test_main_sequence_begin_end_items_post(self):
        round_trip("""
        # C start a
        # C start b
        - abc      # abc comment
        - ghi
        - klm      # klm comment
        # C end a
        # C end b
        """)

    def test_main_mapping_begin_end_complex(self):
        round_trip("""
        # C start a
        # C start b
        abc: 1
        ghi: 2
        klm:
          3a: alpha
          3b: beta   # it is all greek to me
        # C end a
        # C end b
        """)

    def test_09(self):   # 2.9 from the examples in the spec
        round_trip("""
        hr: # 1998 hr ranking
          - Mark McGwire
          - Sammy Sosa
        rbi:
          # 1998 rbi ranking
        - Sammy Sosa
        - Ken Griffey
        """)

    def test_simple_map_middle_comment(self):
        round_trip("""
        abc: 1
        # C 3a
        # C 3b
        ghi: 2
        """)

    def test_map_in_map_0(self):
        round_trip("""
        map1: # comment 1
          # comment 2
          map2:
            key1: val1
        """)

    def test_map_in_map_1(self):
        # comment is moved from value to key
        round_trip("""
        map1:
          # comment 1
          map2:
            key1: val1
        """)

    def test_application_arguments(self):
        # application configur
        round_trip("""
        args:
          username: anthon
          passwd: secret
          fullname: Anthon van der Neut
          tmux:
            session-name: test
          loop:
            wait: 10
        """)

    def test_substitute(self):
        x = dedent("""
        args:
          username: anthon          # name
          passwd: secret            # password
          fullname: Anthon van der Neut
          tmux:
            session-name: test
          loop:
            wait: 10
        """)
        data = round_trip_load(x)
        data['args']['passwd'] = 'deleted password'
        # note the requirement to add spaces for alignment of comment
        x = x.replace(': secret          ', ': deleted password')
        assert round_trip_dump(data) == x


    def test_set_comment(self):
        round_trip("""
        !!set
        # the beginning
        ? a
        # next one is B (lowercase)
        ? b  #  You see? Promised you.
        ? c
        # this is the end
        """)

    @pytest.mark.xfail
    def XXXtest_set_comment_before_tag(self):
        # no comments before tags
        round_trip("""
        # the beginning
        !!set
        # or this one?
        ? a
        # next one is B (lowercase)
        ? b  #  You see? Promised you.
        ? c
        # this is the end
        """)

    def test_omap_comment_roundtrip(self):
        round_trip("""
        !!omap
        - a: 1
        - b: 2  # two
        - c: 3  # three
        - d: 4
        """)

    def test_omap_comment_roundtrip_pre_comment(self):
        round_trip("""
        !!omap
        - a: 1
        - b: 2  # two
        - c: 3  # three
        # last one
        - d: 4
        """)

    @pytest.mark.xfail
    def test_non_ascii_comment(self):
        round_trip("""
        verbosity: 1                  # 0 is minimal output, -1 none
        base_url: http://gopher.net
        special_indices: [1, 5, 8]
        also_special:
        - a
        - 19
        - 32
        asia and europe: &asia_europe
            Turkey: Ankara
            Russia: Moscow
        countries:
            Asia:
                <<: *asia_europe
                Japan: Tokyo # 東京
            Europe:
                <<: *asia_europe
                Spain: Madrid
                Italy: Rome
        """)


class TestMultiLevelGet:
    def test_mlget_00(self):
        x = dedent("""\
        a:
        - b:
          c: 42
        - d:
            f: 196
          e:
            g: 3.14
        """)
        d = round_trip_load(x)
        assert d.mlget(['a', 1, 'd', 'f'], list_ok=True) == 196
        with pytest.raises(AssertionError):
            d.mlget(['a', 1, 'd', 'f']) == 196





import pytest

import ruamel.yaml
from roundtrip import round_trip, dedent


class TestProgramConfig:
    def test_application_arguments(self):
        # application configur
        round_trip("""
        args:
          username: anthon
          passwd: secret
          fullname: Anthon van der Neut
          tmux:
            session-name: test
          loop:
            wait: 10
        """)

    def test_single(self):
        # application configuration
        round_trip("""
        # default arguments for the program
        args:  # needed to prevent comment wrapping
        # this should be your username
          username: anthon
          passwd: secret        # this is plaintext don't reuse \
# important/system passwords
          fullname: Anthon van der Neut
          tmux:
            session-name: test  # make sure this doesn't clash with
                                # other sessions
          loop:   # looping related defaults
            # experiment with the following
            wait: 10
          # no more argument info to pass
        """)

    def test_multi(self):
        # application configuration
        round_trip("""
        # default arguments for the program
        args:  # needed to prevent comment wrapping
        # this should be your username
          username: anthon
          passwd: secret        # this is plaintext don't reuse
                                # important/system passwords
          fullname: Anthon van der Neut
          tmux:
            session-name: test  # make sure this doesn't clash with
                                # other sessions
          loop:   # looping related defaults
            # experiment with the following
            wait: 10
          # no more argument info to pass
        """)



from __future__ import absolute_import
from __future__ import print_function

__all__ = ["ScalarString", "PreservedScalarString"]

try:
    from .compat import text_type
except (ImportError, ValueError):  # for Jython
    from ruamel.yaml.compat import text_type



class ScalarString(text_type):
    def __new__(cls, *args, **kw):
        return text_type.__new__(cls, *args, **kw)


class PreservedScalarString(ScalarString):
    def __new__(cls, value):
        return ScalarString.__new__(cls, value)

def preserve_literal(s):
    return PreservedScalarString(s.replace('\r\n', '\n').replace('\r', '\n'))


def walk_tree(base):
    """
    the routine here walks over a simple yaml tree (recursing in
    dict values and list items) and converts strings that
    have multiple lines to literal scalars
    """
    from ruamel.yaml.compat import string_types


    if isinstance(base, dict):
        for k in base:
            v = base[k]
            if isinstance(v, string_types) and '\n' in v:
                base[k] = preserve_literal(v)
            else:
                walk_tree(v)
    elif isinstance(base, list):
        for idx, elem in enumerate(base):
            if isinstance(elem, string_types) and '\n' in elem:
                print(elem)
                base[idx] = preserve_literal(elem)
            else:
                walk_tree(elem)




from roundtrip import dedent


class TestDedent:
    def test_start_newline(self):
        x = dedent("""
        123
          456
        """)
        assert x == "123\n  456\n"

    def test_start_space_newline(self):
        # special construct to prevent stripping of following whitespac
        x = dedent("   " """
        123
        """)
        assert x == "123\n"

    def test_start_no_newline(self):
        # special construct to prevent stripping of following whitespac
        x = dedent("""\
        123
          456
        """)
        assert x == "123\n  456\n"

    def test_preserve_no_newline_at_end(self):
        x = dedent("""
        123""")
        assert x == "123"

    def test_preserve_no_newline_at_all(self):
        x = dedent("""\
        123""")
        assert x == "123"

    def test_multiple_dedent(self):
        x = dedent(dedent("""
        123
        """))
        assert x == "123\n"




from __future__ import print_function

"""
The windows whl file has no C stuff but a
ruamel.yaml-0.9.2-py2-none-any.whl file overrules the .tar.gz on Linux.

You can create a .whl and copy it to impure names (or start
with an impure one), not sure if this is necessary.

"""

import sys
import os
import shutil


def main():
    src = sys.argv[1]
    print(src, '-->')
    dir_name = os.path.dirname(src)
    base_name = os.path.basename(src)
    p, v, rest = base_name.split('-', 2)
    # print dir_name
    for pyver in ['cp26', 'cp27', 'cp33', 'cp34']:
        for platform in ['win32', 'win_amd64']:
            dst = os.path.join(dir_name,
                               '%s-%s-%s-none-%s.whl' % (
                                   p, v, pyver, platform
                               ))
            print(dst)
            shutil.copy(src, dst)

main()





def configobj_walker(cfg):
    """
    walks over a ConfigObj (INI file with comments) generating
    corresponding YAML output (including comments
    """
    from configobj import ConfigObj
    assert isinstance(cfg, ConfigObj)
    for c in cfg.initial_comment:
        if c.strip():
            yield c
    for s in _walk_section(cfg):
        if s.strip():
            yield s
    for c in cfg.final_comment:
        if c.strip():
            yield c


def _walk_section(s, level=0):
    from configobj import Section
    assert isinstance(s, Section)
    indent = u'  ' * level
    for name in s.scalars:
        for c in s.comments[name]:
            yield indent + c.strip()
        x = s[name]
        if u'\n' in x:
            i = indent + u'  '
            x = u'|\n' + i + x.strip().replace(u'\n', u'\n' + i)
        elif ':' in x:
            x = u"'" + x.replace(u"'", u"''") + u"'"
        line = u'{0}{1}: {2}'.format(indent, name, x)
        c = s.inline_comments[name]
        if c:
            line += u' ' + c
        yield line
    for name in s.sections:
        for c in s.comments[name]:
            yield indent + c.strip()
        line = u'{0}{1}:'.format(indent, name)
        c = s.inline_comments[name]
        if c:
            line += u' ' + c
        yield line
        for val in _walk_section(s[name], level=level+1):
            yield val

##def config_obj_2_rt_yaml(cfg):
##    from .comments import CommentedMap, CommentedSeq
##    from configobj import ConfigObj
##    assert isinstance(cfg, ConfigObj)
##    #for c in cfg.initial_comment:
##    #    if c.strip():
##    #        pass
##    cm = CommentedMap()
##    for name in s.sections:
##        cm[name] = d = CommentedMap()
##
##
##    #for c in cfg.final_comment:
##    #    if c.strip():
##    #        yield c
##    return cm




import platform
import pytest

@pytest.mark.skipif(platform.python_implementation() == 'Jython',
                reason="Jython throws RepresenterError")
def test_load_cyaml():
    import ruamel.yaml
    assert ruamel.yaml.__with_libyaml__
    from ruamel.yaml.cyaml import CLoader
    ruamel.yaml.load("abc: 1", Loader=CLoader)




from __future__ import print_function

"""
various test cases for YAML files
"""

import pytest
import platform

import ruamel.yaml
from ruamel.yaml.compat import ordereddict
from roundtrip import round_trip, dedent, round_trip_load, round_trip_dump


class TestYAML:
    def test_backslash(self):
        round_trip("""
        handlers:
          static_files: applications/\\1/static/\\2
        """)

    def test_omap_out(self):
        # ordereddict mapped to !!omap
        x = ordereddict([('a', 1), ('b', 2)])
        res = ruamel.yaml.dump(x, default_flow_style=False)
        assert res == dedent("""
        !!omap
        - a: 1
        - b: 2
        """)

    def test_omap_roundtrip(self):
        round_trip("""
        !!omap
        - a: 1
        - b: 2
        - c: 3
        - d: 4
        """)

    def test_CommentedSet(self):
        from ruamel.yaml.constructor import CommentedSet
        s = CommentedSet(['a', 'b', 'c'])
        s.remove('b')
        s.add('d')
        assert s == CommentedSet(['a', 'c', 'd'])
        s.add('e')
        s.add('f')
        s.remove('e')
        assert s == CommentedSet(['a', 'c', 'd', 'f'])

    def test_set_out(self):
        # preferable would be the shorter format without the ': null'
        from ruamel.yaml.compat import ordereddict
        x = set(['a', 'b', 'c'])
        res = ruamel.yaml.dump(x, default_flow_style=False)
        assert res == dedent("""
        !!set
        a: null
        b: null
        c: null
        """)

    # @pytest.mark.xfail
    # ordering is not preserved in a set
    def test_set_compact(self):
        # this format is read and also should be written by default
        round_trip("""
        !!set
        ? a
        ? b
        ? c
        """)

    def test_blank_line_after_comment(self):
        round_trip("""
        # Comment with spaces after it.


        a: 1
        """)

    def test_blank_line_between_seq_items(self):
        round_trip("""
        # Seq with spaces in between items.
        b:
        - bar


        - baz
        """)

    @pytest.mark.skipif(platform.python_implementation() == 'Jython',
                    reason="Jython throws RepresenterError")
    def test_blank_line_after_literal_chip(self):
        s = """
        c:
        - |
          This item
          has a blank line
          following it.

        - |
          To visually separate it from this item.

          This item contains a blank line.


        """
        d = round_trip_load(dedent(s))
        print(d)
        round_trip(s)
        assert d['c'][0].split('it.')[1] == '\n'
        assert d['c'][1].split('line.')[1] == '\n'

    @pytest.mark.skipif(platform.python_implementation() == 'Jython',
                    reason="Jython throws RepresenterError")
    def test_blank_line_after_literal_keep(self):
        """ have to insert an eof marker in YAML to test this"""
        s = """
        c:
        - |+
          This item
          has a blank line
          following it.

        - |+
          To visually separate it from this item.

          This item contains a blank line.


        ...
        """
        d = round_trip_load(dedent(s))
        print(d)
        round_trip(s)
        assert d['c'][0].split('it.')[1] == '\n\n'
        assert d['c'][1].split('line.')[1] == '\n\n\n'

    @pytest.mark.skipif(platform.python_implementation() == 'Jython',
                    reason="Jython throws RepresenterError")
    def test_blank_line_after_literal_strip(self):
        s = """
        c:
        - |-
          This item
          has a blank line
          following it.

        - |-
          To visually separate it from this item.

          This item contains a blank line.


        """
        d = round_trip_load(dedent(s))
        print(d)
        round_trip(s)
        assert d['c'][0].split('it.')[1] == ''
        assert d['c'][1].split('line.')[1] == ''

